#!/usr/bin/env python
"""
Example: simple line plot.
Show how to make and save a simple line plot with labels, title and grid
"""
from pylab import *

figure()
t = arange(0.0, 1.0+0.01, 0.01)
s = cos(2*2*pi*t)
plot(t, s, '-', lw=2)

xlabel('time (s)')
ylabel('voltage (mV)')
title('About as simple as it gets, folks')
grid(True)
show()



#!/usr/bin/env python3
##  @file
#   @brief Example Script simple session
#   @ingroup python_bindings_examples

from gnucash import (
     Session, GnuCashBackendException,
     SessionOpenMode,
     ERR_BACKEND_LOCKED, ERR_FILEIO_FILE_NOT_FOUND
)

FILE_1 = "/tmp/not_there.xac"
FILE_2 = "/tmp/example_file.xac"

# open a file that isn't there, detect the error
session = None
try:
    session = Session(FILE_1)
except GnuCashBackendException as backend_exception:
    assert( ERR_FILEIO_FILE_NOT_FOUND in backend_exception.errors)


# create a new file, this requires a file type specification
with Session("xml://%s" % FILE_2, SessionOpenMode.SESSION_NEW_STORE) as session:
    book = session.book
    root = book.get_root_account()

# open the new file, try to open it a second time, detect the lock
with Session(FILE_2) as session:
    try:
        session_2 = Session(FILE_2)
    except GnuCashBackendException as backend_exception:
        assert( ERR_BACKEND_LOCKED in backend_exception.errors )



from unittest import TestCase, main

from gnucash import Session

class BookSession(TestCase):
    def setUp(self):
        self.ses = Session()
        self.book = self.ses.get_book()
        self.table = self.book.get_table()
        self.currency = self.table.lookup('CURRENCY', 'EUR')

class TestBook(BookSession):
    def test_markclosed(self):
        self.ses.end()

if __name__ == '__main__':
    main()



#!/usr/bin/env python3
## @file
# @brief Creates a basic set of accounts and a couple of transactions
# @ingroup python_bindings_examples

from gnucash import (
        Session, Account, Transaction, Split, GncNumeric, SessionOpenMode)

FILE_1 = "/tmp/example.gnucash"

with Session("xml://%s" % FILE_1, SessionOpenMode.SESSION_NEW_STORE) as session:

    book = session.book
    root_acct = Account(book)
    expenses_acct = Account(book)
    savings_acct = Account(book)
    opening_acct = Account(book)
    trans1 = Transaction(book)
    trans1.BeginEdit()
    trans2 = Transaction(book)
    trans2.BeginEdit()

    split1 = Split(book)
    split3 = Split(book)
    comm_table = book.get_table()
    cad = comm_table.lookup("CURRENCY", "CAD")

    num1 = GncNumeric(4, 1)
    num2 = GncNumeric(100, 1)

    #Set new root account
    book.set_root_account(root_acct)

    #Set up root account and add sub-accounts
    root_acct.SetName("Root")
    root_acct.SetType(13) #ACCT_TYPE_ROOT = 13
    root_acct.append_child(expenses_acct)
    root_acct.append_child(savings_acct)
    root_acct.append_child(opening_acct)

    #Set up Expenses account
    expenses_acct.SetCommodity(cad)
    expenses_acct.SetName("Expenses")
    expenses_acct.SetType(9) #ACCT_TYPE_EXPENSE = 9

    #Set up Savings account
    savings_acct.SetCommodity(cad)
    savings_acct.SetName("Savings")
    savings_acct.SetType(0) #ACCT_TYPE_BANK = 0

    #Set up Opening Balance account
    opening_acct.SetCommodity(cad)
    opening_acct.SetName("Opening Balance")
    opening_acct.SetType(10) #ACCT_TYPE_EQUITY = 10

    split1.SetValue(num1)
    split1.SetAccount(expenses_acct)
    split1.SetParent(trans1)

    split3.SetValue(num2)
    split3.SetAccount(savings_acct)
    split3.SetParent(trans2)

    trans1.SetCurrency(cad)
    trans1.SetDate(14, 3, 2006)
    trans1.SetDescription("Groceries")

    trans2.SetCurrency(cad)
    trans2.SetDate(7, 11, 1995)
    trans2.SetDescription("Opening Savings Balance")

    split2 = Split(book)
    split2.SetAccount(savings_acct)
    split2.SetParent(trans1)
    split2.SetValue(num1.neg())

    split4 = Split(book)
    split4.SetAccount(opening_acct)
    split4.SetParent(trans2)
    split4.SetValue(num2.neg())


    trans1.CommitEdit()
    trans2.CommitEdit()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

##@file
# @brief Exports an invoice to lco-file for use with LaTeX, see \ref py_invoice_export
# @ingroup python_bindings_examples
# @author Christoph Holtermann (c.holtermann (at) gmx.de)
# @date May 2011
#
# @details
# The output file can be imported into KOMA-Script-letters.
# This works primarily for germany. Internationalization welcome!
#
# Additional files:
#
# - Invoice.tex\n
# Example template file. Should be modified according to personal needs.
# - rechnung.sty\n
# style file for invoices.\n
# This file is not part of the python-bindings!\n
# For an example where to get it see section credits below.
#
# Usage :
# \code latex_invoice -l -f -n INVOICE_NUMBER file://testfile \endcode
# will create file data.lco.
# \code latex --output-format=pdf Invoice.tex \endcode
# should run latex on file Invoice.tex and result in Invoice.pdf. Invoice.tex includes data.lco.
#
# Additional information :
#
# - Doxygen docs: see page \ref py_invoice_export at https://code.gnucash.org/docs/MAINT or MASTER
# - https://www.uweziegenhagen.de/latex/documents/rechnung/rechnungen.pdf (german)
#
# Credits to and ideas from
#
# - Main function as proposed by Guido van Rossum
#   at https://www.artima.com/weblogs/viewpost.jsp?thread=4829
# - Invoice.tex is derived from\n
#   scrlttr2.tex v0.3. (c) by Juergen Fenn <juergen.fenn@gmx.de>\n
#   https://www.komascript.de/node/355\n
#   english translation: ftp://ftp.dante.de/tex-archive/info/templates/fenn/scrlttr2en.tex
# - rechnung.sty\n
#   from M G Berberich (berberic@fmi.uni-passau.de) and Ulrich Sibiller (uli42@web.de)
#   Ver3.10 from https://www.forwiss.uni-passau.de/~berberic/TeX/Rechnung/index.html
#
# To Do:
#
# - get own contact data from gnucash
# - have own bank information in footline
# - nicer formatting of invoice date and date due
# - is there anything else missing in this invoice ?

try:
    import sys
    import getopt
    import gnucash
    import str_methods
    from gncinvoicefkt import *
    from IPython import version_info as IPython_version_info

    if IPython_version_info[0] >= 1:
        from IPython.terminal.ipapp import TerminalIPythonApp
    else:
        from IPython.frontend.terminal.ipapp import TerminalIPythonApp
    from gnucash.gnucash_business import (
        Customer,
        Employee,
        Vendor,
        Job,
        Address,
        Invoice,
        Entry,
        TaxTable,
        TaxTableEntry,
        GNC_AMT_TYPE_PERCENT,
        GNC_DISC_PRETAX,
    )
    from gnucash import SessionOpenMode
    import locale
except ImportError as import_error:
    print("Problem importing modules.")
    print(import_error)
    sys.exit(2)


class Usage(Exception):
    def __init__(self, msg):
        self.msg = msg


def invoice_to_lco(invoice):
    """returns a string which forms a lco-file for use with LaTeX"""

    lco_out = u"\ProvidesFile{data.lco}[]\n"

    def write_variable(ukey, uvalue, replace_linebreak=True):

        outstr = u""
        if uvalue.endswith("\n"):
            uvalue = uvalue[0 : len(uvalue) - 1]

        if not ukey in [u"fromaddress", u"toaddress", u"date"]:
            outstr += u"\\newkomavar{"
            outstr += ukey
            outstr += u"}\n"

        outstr += u"\\setkomavar{"
        outstr += ukey
        outstr += u"}{"
        if replace_linebreak:
            outstr += uvalue.replace(u"\n", u"\\\\") + "}"
        return outstr

    # Write owners address
    add_str = u""
    owner = invoice.GetOwner()
    if owner.GetName() != "":
        add_str += owner.GetName() + "\n"

    addr = owner.GetAddr()
    if addr.GetName() != "":
        add_str += addr.GetName() + "\n"
    if addr.GetAddr1() != "":
        add_str += addr.GetAddr1() + "\n"
    if addr.GetAddr2() != "":
        add_str += addr.GetAddr2() + "\n"
    if addr.GetAddr3() != "":
        add_str += addr.GetAddr3() + "\n"
    if addr.GetAddr4() != "":
        add_str += addr.GetAddr4() + "\n"

    lco_out += write_variable("toaddress2", add_str)

    # Invoice number
    inr_str = invoice.GetID()
    lco_out += write_variable("rechnungsnummer", inr_str)

    # date
    date = invoice.GetDatePosted()
    udate = date.strftime("%d.%m.%Y")
    lco_out += write_variable("date", udate) + "\n"

    # date due
    date_due = invoice.GetDateDue()
    udate_due = date_due.strftime("%d.%m.%Y")
    lco_out += write_variable("date_due", udate_due) + "\n"

    # Write the entries
    ent_str = u""
    locale.setlocale(locale.LC_ALL, "")
    for n, ent in enumerate(invoice.GetEntries()):

        line_str = u""

        if type(ent) != Entry:
            ent = Entry(instance=ent)  # Add to method_returns_list

        descr = ent.GetDescription()
        price = ent.GetInvPrice().to_double()
        n = ent.GetQuantity()

        uprice = locale.currency(price).rstrip(" EUR")
        un = str(
            int(float(n.num()) / n.denom())
        )  # choose best way to format numbers according to locale

        line_str = u"\Artikel{"
        line_str += un
        line_str += u"}{"
        line_str += descr
        line_str += u"}{"
        line_str += uprice
        line_str += u"}"

        # print(line_str)
        ent_str += line_str

    lco_out += write_variable("entries", ent_str)

    return lco_out


def main(argv=None):
    if argv is None:
        argv = sys.argv
    try:
        prog_name = argv[0]
        with_ipshell = False
        ignore_lock = False
        no_latex_output = True
        list_invoices = False
        output_file_name = "data.lco"
        invoice_number = None

        try:
            opts, args = getopt.getopt(argv[1:], "fhiln:po:", ["help"])
        except getopt.error as msg:
            raise Usage(msg)

        for opt in opts:
            if opt[0] in ["-f"]:
                print("ignoring lock")
                ignore_lock = True
            if opt[0] in ["-h", "--help"]:
                raise Usage("Help:")
            if opt[0] in ["-i"]:
                print("Using ipshell")
                with_ipshell = True
            if opt[0] in ["-l"]:
                print("listing all invoices")
                list_invoices = True
            if opt[0] in ["-n"]:
                invoice_number = int(opt[1])
                print("using invoice number", invoice_number)
                no_latex_output = False
            if opt[0] in ["-o"]:
                output_file_name = opt[1]
                print("using output file", output_file_name)
        if len(args) > 1:
            print("opts:", opts, "args:", args)
            raise Usage("Only one input can be accepted !")
        if len(args) == 0:
            raise Usage("No input given !")
        input_url = args[0]
    except Usage as err:
        if err.msg == "Help:":
            retcode = 0
        else:
            print("Error:", err.msg, file=sys.stderr)
            print("for help use --help", file=sys.stderr)
            retcode = 2

        print("Generate a LaTeX invoice or print out all invoices.")
        print()
        print("Usage:")
        print()
        print("Invoke with", prog_name, "input.")
        print("where input is")
        print("   filename")
        print("or file://filename")
        print("or mysql://user:password@host/databasename")
        print()
        print("-f             force open = ignore lock")
        print("-h or --help   for this help")
        print("-i             for ipython shell")
        print("-l             list all invoices")
        print("-n number      use invoice number (no. from previous run with -l)")
        print("-o name        use name as outputfile. default: data.lco")

        return retcode

    # Try to open the given input
    try:
        session = gnucash.Session(
            input_url,
            SessionOpenMode.SESSION_READ_ONLY
            if ignore_lock
            else SessionOpenMode.SESSION_NORMAL_OPEN,
        )
    except Exception as exception:
        print("Problem opening input.")
        print(exception)
        return 2

    book = session.book
    root_account = book.get_root_account()
    comm_table = book.get_table()
    EUR = comm_table.lookup("CURRENCY", "EUR")

    invoice_list = get_all_invoices(book)

    if list_invoices:
        for number, invoice in enumerate(invoice_list):
            print(str(number) + ")")
            print(invoice)

    if not (no_latex_output):

        if invoice_number == None:
            print("Using the first invoice:")
            invoice_number = 0

        invoice = invoice_list[invoice_number]
        print("Using the following invoice:")
        print(invoice)

        lco_str = invoice_to_lco(invoice)

        # Opening output file
        f = open(output_file_name, "w")
        f.write(lco_str)
        f.close()

    if with_ipshell:
        app = TerminalIPythonApp.instance()
        app.initialize(argv=[])  # argv=[] instructs IPython to ignore sys.argv
        app.start()

    # session.save()
    session.end()


if __name__ == "__main__":
    sys.exit(main())



# test gettext
#
# @date 2020-04-08
# @author Christoph Holtermann <mail@c-holtermann.net>

from unittest import TestCase, main
import gnucash
from gnucash import _sw_core_utils


class TestGettext(TestCase):
    def test_import_gettext(self):
        import gettext

    def test_get_localedir(self):
        _localedir = _sw_core_utils.gnc_path_get_localedir()

if __name__ == '__main__':
    main()
